#------------------------------------------------------------------------------
# Modul            : G r a p h . p m
#------------------------------------------------------------------------------
# Autor            : Christian Dhl
# Erstellt         : 08. Februar 2004
# Zuletzt gendert : 18. Januar 2005
# Aufgabe          : Logische Graphenklasse (ohne Darstellungselemente).
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# Globale private Datei-Variablen:
#------------------------------------------------------------------------------
my $Debug  = 0;


#------------------------------------------------------------------------------
# Klasse           : G r a p h : : G E D : : G r a p h
# abgeleitet von   : Graph::GED::Basis
#------------------------------------------------------------------------------
# Autor            : Christian Dhl
# Beschreibung     : Eine Graphen-Klasse.
# Attribute        : n           - Anzahl der Ecken
#                    m           - Anzahl der Kanten
#                    kantenform  - Art der Kante (normal, Pfeil hin...)
#                    _kanten     - anonymes Array von Kante-Objekten
# Klassen-Methoden : new         - Erstellt ein Objekt dieser Klasse.
# Objekt-Methoden  : clone             - Erzeugt ein geclontes Objekt (eine
#                                        Kopie, die referenzierte Memberhash-
#                                        werte richtig kopiert und nicht nur
#                                        eine Kopie der Referenz anlegt.
#                    n                 - Getter / Setter
#                    m                 - Getter / Setter
#                    kantenform        - Getter / Setter
#                    _kanten           - Interner Getter / Setter
#                    setze_kanten      - Ersetzt die Kantenmenge
#                    loesche_kanten    - Lscht die Kantenmenge
#                    kante             - Getter/Setter ber Index fr
#                                        eine Kante aus der Menge
#                    entnehme_kante    - spliced eine Kante heraus
#                    fuege_kante_hinzu - pushed eine Kante hinzu
#                    ecke_einfuegen    - Ecke einfgen
#                    ecke_loeschen     - Ecke lschen
#                    ecke_mergen       - Ecke mergen (Ecke wird gelscht
#                                        und die Nachbarn der Ecke alle
#                                        miteinander durch Kanten verbunden).
#                    kante_existiert   - Testet, ob die beiden angegeben Ecken
#                                        durch eine Kante verbunden sind.
#                    kante_einfuegen   - Fgt eine Kante ein.
#                    kante_loeschen    - Lscht eine Kante.
#                    clear             - Lscht alles.
#------------------------------------------------------------------------------
# 0.0.1 - 08.02.2004 - CD - erstellt
#------------------------------------------------------------------------------
package Graph::GED::Graph;

use strict;
use warnings;

use base qw/Graph::GED::Basis/;

use Data::Dumper;
use Graph::GED::Kante;

sub new {
    #--------------------------------------------------------------------------
    # Methode      : n e w
    # Klasse       : Graph::GED::Graph
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Erzeugt ein neues Graph-Objekt.
    # Parameter    : Klasse
    # Rckgabewert : Das geblesste neue Objekt.
    #--------------------------------------------------------------------------
    # 0.0.1 - 08.02.2004 - CD - erstellt
    # 0.0.2 - 10.02.2004 - CD - Vereinfachung durch Verwendung der Klasse Basis.
    #--------------------------------------------------------------------------

    my $self  = shift;
    my $class = (ref $self) || $self;

    #--------------------------------------------------------------------------
    # Ausgabe der Aufrufparameter:
    #--------------------------------------------------------------------------
    print "Graph::GED::Graph::new\n",
          "\tself       = '$self'\n",
          "\tclass      = '$class'\n",
          "\tParameter  = @_\n",
          if $Debug > 1;


    $self = $class->SUPER::new();


    #--------------------------------------------------------------------------
    # Nun setzen wir noch ein paar Attribute:
    #--------------------------------------------------------------------------
    $self->n         (0);
    $self->m         (0);
    $self->kantenform('');
    $self->_kanten   ([]);

    #--------------------------------------------------------------------------
    # ... und geben das erzeugte Objekt zurck:
    #--------------------------------------------------------------------------
    return $self;

} # sub Graph::new


sub clone {
    #--------------------------------------------------------------------------
    # Methode      : c l o n e
    # Klasse       : Graph::GED::Graph
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Erzeugt ein geclontes Objekt (eine Kopie, die referenzier-
    #                te Memberhashwerte richtig kopiert und nicht nur eine
    #                Kopie der Referenz anlegt.
    # Parameter    : Ev. ein von einer abgeleiteten Klasse nach unten durchge-
    #                reichtes Objekt, das wir wiederum durchreichen sollen und
    #                anschlieend seine in dieser Klasse hinzugefgten Kon-
    #                tainer kopieren sollen.
    # Rckgabewert : Das geclonte Objekt.
    #--------------------------------------------------------------------------
    # 0.0.1 - 10.02.2004 - CD - erstellt
    # 0.0.2 - 11.02.2004 - CD - Funktionalitt fertig gestellt
    #--------------------------------------------------------------------------

    my $self = shift;
    my $ursp = shift;
    my $clone;

    print "Graph::GED::Graph::Clone\n" if $Debug;

    #--------------------------------------------------------------------------
    # Bekommen wir von "oben" ein Objekt?
    #--------------------------------------------------------------------------
    if (defined $ursp) {
        $clone = $ursp;
    }
    #--------------------------------------------------------------------------
    # Falls nicht erzeugen wir das Objekt und kopieren unsere Daten flach:
    #--------------------------------------------------------------------------
    else {
        $clone = $self->new();
        %$clone = %$self;
    }

    #--------------------------------------------------------------------------
    # Geerbte Teile clonen:
    #--------------------------------------------------------------------------
    $self->SUPER::clone($clone);

    #--------------------------------------------------------------------------
    # Dann Kontainer kopieren:
    #--------------------------------------------------------------------------
    # (Hier mssen auch alle Elemente des Arrays gecloned werden, da das ja
    # auch Referenzen sind!)
    #--------------------------------------------------------------------------
    my @kanten;
    for (@{$self->_kanten()}) {
        my $kante = $_->clone();
        print "Originalkante: ", $_    , "\n",
              "clonekante   : ", $kante, "\n"
              if $Debug > 1;
        push @kanten, $kante;
    }
    #$clone->_kanten(\@kanten);        # Referenz auf Kopie ablegen
    $clone->_kanten([ @kanten ]);        # Referenz auf Kopie ablegen


    return $clone;
} # sub clone


#------------------------------------------------------------------------------
# The getter/setter methods:
#------------------------------------------------------------------------------
sub n          { return shift->_property('n',          @_) }
sub m          { return shift->_property('m',          @_) }
sub kantenform { return shift->_property('kantenform', @_) }
sub _kanten    { return shift->_property('_kanten',    @_) }

#------------------------------------------------------------------------------
# Hilfsfunktionen fr n und m:
#------------------------------------------------------------------------------
sub inc_n { $_[0]->n($_[0]->n()+1) } # n bzw. m
sub dec_n { $_[0]->n($_[0]->n()-1) } # in-
sub inc_m { $_[0]->m($_[0]->m()+1) } # bzw.
sub dec_m { $_[0]->m($_[0]->m()-1) } # decrementieren



#==============================================================================
#
#
#   H I L F S M E T H O D E N
#
#   (nur diese verwenden _kanten)
#
#==============================================================================


sub setze_kanten {
    #--------------------------------------------------------------------------
    # Methode      : s e t z e _ k a n t e n
    # Klasse       : Graph::GED::Graph
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Ersetzt die gesamte Kantenmenge.
    #                Die alte Menge wird vorher durch das Setzen auf undef
    #                freigegeben.
    # Parameter    : Referenz auf Array (mit Kante-Objekten)
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 10.02.2004 - CD - erstellt
    #--------------------------------------------------------------------------

    $_[0]->_kanten(undef);
    $_[0]->_kanten($_[1]);

} # sub setze_kanten


sub loesche_kanten {
    #--------------------------------------------------------------------------
    # Methode      : l o e s c h e _ k a n t e n
    # Klasse       : Graph::GED::Graph
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Setzt die gesamte Kantenmenge auf eine leere Menge.
    #                Die alte Menge wird vorher durch das Setzen auf undef
    #                freigegeben.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 10.02.2004 - CD - erstellt
    #--------------------------------------------------------------------------

    $_[0]->_kanten(undef);
    $_[0]->_kanten([]);

} # sub loesche_kanten


sub kante {
    #--------------------------------------------------------------------------
    # Methode      : k a n t e
    # Klasse       : Graph::GED::Graph
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Setter / Getter fr ein Kante(nobjekt)
    # Parameter    : 1) Index
    #                2) Wert (optional)
    # Rckgabewert : Der aktuelle Wert (im Falle des Getters) oder der alte
    #               Wert (im falle des Setters).
    #--------------------------------------------------------------------------
    # 0.0.1 - 10.02.2004 - CD - erstellt
    #--------------------------------------------------------------------------

    if (@_ == 3) {
        my $aw = $_[0]->_kanten()->[$_[1]];
        $_[0]->_kanten()->[$_[1]] = $_[2];
        return $aw;
    }
    else {
        return $_[0]->_kanten()->[$_[1]];
    }

} # sub kante


sub entnehme_kante {
    #--------------------------------------------------------------------------
    # Methode      : e n t n e h m e _ k a n t e
    # Klasse       : Graph::GED::Graph
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Entfernt eine Kante aus der Kantenmenge und gibt diese
    #                zurck.
    # Parameter    : Index
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 10.02.2004 - CD - erstellt
    #--------------------------------------------------------------------------

    return splice @{$_[0]->_kanten()}, $_[1], 1;

} # sub entnehme_kante


sub fuege_kante_hinzu {
    #--------------------------------------------------------------------------
    # Methode      : f u e g e _ k a n t e _ h i n z u
    # Klasse       : Graph::GED::Graph
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Fgt eine Kante zur Kantenmenge hinzu.
    # Parameter    : Kante-Objekt
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 10.02.2004 - CD - erstellt
    #--------------------------------------------------------------------------

    push @{$_[0]->_kanten()}, $_[1];

} # sub fuege_kante_hinzu



#==============================================================================
#
#
#   M E T H O D E N
#
#
#==============================================================================


sub ecke_einfuegen {
    #--------------------------------------------------------------------------
    # Methode      : e c k e _ e i n f u e g e n
    # Klasse       : Graph::GED::Graph
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Fgt eine neue Ecke in den Graphen ein.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 08.02.2004 - CD - erstellt
    # 0.0.2 - 10.02.2004 - CD - inc_n verwendet
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph::ecke_einfuegen mit Parametern aufgerufen!" if scalar @_;

    print "Graph::GED::Graph::ecke_einfuegen\n" if $Debug;

    #--------------------------------------------------------------------------
    # Eckenzhler incrementieren
    #--------------------------------------------------------------------------
    $self->inc_n();

} # sub Graph::ecke_einfuegen


sub ecke_loeschen {
    #--------------------------------------------------------------------------
    # Methode      : e c k e _ l o e s c h e n
    # Klasse       : Graph::GED::Graph
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Lscht die angegebene Ecke mit allen inzidierenden Kanten
    #                aus dem Graphen.
    # Parameter    : zu lschende Ecke
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 08.02.2004 - CD - erstellt
    # 0.0.2 - 10.02.2004 - CD - neue Hilfsmethoden verwendet.
    #                           Umstellung auf die Kantenklasse.
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph::ecke_loeschen falscher Anzahl Parametern " .
        "aufgerufen!"
        if scalar @_ != 1;
    die "Graph::GED::Graph::ecke_loeschen aufgerufen, aber der Graph hat " .
        "keine Ecken!"
        unless $self->n() > 0;

    my $e = shift;


    print "Graph::GED::Graph::ecke_loeschen($e)\n" if $Debug;

    die "Graph::GED::Graph::ecke_loeschen mit falschem Eckenwert '$e' " .
        "aufgerufen, n ist " .
        $self->n() ."!"
        if $e > $self->n();

    die "Interner Fehler!" if scalar @{$self->_kanten()} != $self->m();

    print Dumper($self) if $Debug > 1;

    #--------------------------------------------------------------------------
    # Kanten raussuchen, die zu dieser Ecke gehren und lschen:
    #--------------------------------------------------------------------------
    for my $i (reverse 0..$self->m()-1) {
        if ($self->kante($i)->inzidiert_mit_ecke($e)) {
            $self->kante_loeschen($i);
        }
    }

    #--------------------------------------------------------------------------
    # Eckenzhler decrementieren
    #--------------------------------------------------------------------------
    $self->dec_n();

    #--------------------------------------------------------------------------
    # Korrigieren der hheren Eckennummern:
    #--------------------------------------------------------------------------
    for my $i (0..$self->m()-1) {
        my $k = $self->kante($i);
        $k->a($k->a()-1) if $k->a() >= $e;
        $k->e($k->e()-1) if $k->e() >= $e;
    }

    print Dumper($self) if $Debug > 1;

} # sub ecke_loeschen


sub ecke_mergen {
    #--------------------------------------------------------------------------
    # Methode      : e c k e _ m e r g e n
    # Klasse       : Graph::GED::Graph
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Mergt die angegebene Ecke (d.h. die Ecke wird gelscht
    #               und die Kanten zusammengezogen.
    # Parameter    : Zu mergende Ecke
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 08.02.2004 - CD - erstellt
    # 0.0.2 - 10.02.2004 - CD - neue Hilfsmethoden verwendet.
    #                           Umstellung auf die Kantenklasse.
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph::ecke_mergen mit falscher Anzahl Parametern " . 
        "aufgerufen!"
        if scalar @_ != 1;

    my $e = shift;

    print "Graph::GED::Graph::ecke_mergen($e)\n" if $Debug;

    die "Interner Fehler!" if scalar @{$self->_kanten()} != $self->m();

    #--------------------------------------------------------------------------
    # Ecke mergen:
    #--------------------------------------------------------------------------
    print Dumper($self) if $Debug > 1;

    #--------------------------------------------------------------------------
    # Kanten raussuchen, die zu dieser Ecke gehren:
    #--------------------------------------------------------------------------
    my @ecken = (); # Nachbarn von $e, die dann verbunden werden mssen
    my @kform = (); # Kantenformen, im Moment ignoriert

    for my $i (reverse 0..$self->m()-1) {
        if ($self->kante($i)->inzidiert_mit_ecke($e)) {
            my $k = $self->entnehme_kante($i);

            print "loesche Kante $i ", $k->a(), " --- ", $k->e(), "\n"
                  if $Debug>1;

            push @ecken, ($k->a()!=$e ? $k->a() : $k->e());
            push @kform, $k->form();
            # Bei der Kantenform msste man noch bercksichtigen, welche Ecke
            # die Kante $e war. Und dann bei Kantenpaaren mit der selben
            # Ausprgung diese wieder verwenden. Im Moment wird alles als
            # normale Kante erzeugt.

            $self->dec_m();
        }
    }

    print "Nachbarn der geloeschten Ecke: @ecken\n" if $Debug;

    #--------------------------------------------------------------------------
    # Neue Kanten eintragen:
    #--------------------------------------------------------------------------
    for my $i1 (0..$#ecken-1) {
        print "i1=$i1\n" if $Debug>1;
        for my $i2 ($i1+1..$#ecken) {
            print "i2=$i2\n" if $Debug>1;
            print "fuege Kante ein: $i1 --- $i2\n" if $Debug;
            $self->kante_einfuegen($ecken[$i1], $ecken[$i2])
                unless $self->kante_existiert($ecken[$i1], $ecken[$i2]);
        }
    }

    #--------------------------------------------------------------------------
    # Ecke lschen:
    #--------------------------------------------------------------------------
    $self->dec_n();

    #--------------------------------------------------------------------------
    # Korrigieren der hheren Eckennummern:
    #--------------------------------------------------------------------------
    for my $i (0..$self->m()-1) {
        my $k = $self->kante($i);
        $k->a($k->a()-1) if $k->a() >= $e;
        $k->e($k->e()-1) if $k->e() >= $e;
    }

    print Dumper($self) if $Debug > 1;

} # sub Graph::ecke_mergen


sub kante_existiert {
    #--------------------------------------------------------------------------
    # Methode      : k a n t e _ e x i s t i e r t
    # Klasse       : Graph::GED::Graph
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Testet, ob die angegebene Kante existiert
    # Parameter    : 1) Ecke Anfang
    #                2) Ecke Ende
    # Rckgabewert : undef falls nicht vorhanden, anderenfalls Kantentyp
    #--------------------------------------------------------------------------
    # 0.0.1 - 08.02.2004 - CD - erstellt
    # 0.0.2 - 10.02.2004 - CD - neue Hilfsmethoden verwendet.
    #                           Umstellung auf die Kantenklasse.
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph::kante_existiert mit falscher Anzahl Parametern " . 
        "aufgerufen!"
        if scalar @_ != 2;

    my $ea   = shift;
    my $ee   = shift;

    my $form = undef;

    print "Graph::GED::Graph::kante_existiert($ea, $ee)\n" if $Debug;

    die "Interner Fehler!" if scalar @{$self->_kanten()} != $self->m();

    for my $i (0..$self->m()-1) {
        my $k = $self->kante($i);
        if (
            $ea == $k->a() and $ee == $k->e() or
            $ea == $k->e() and $ee == $k->a()
           )
        {
            $form = $k->form();
            last;
        }
    }


    return $form;

} # sub Graph::kante_existiert


sub kante_einfuegen {
    #--------------------------------------------------------------------------
    # Methode      : k a n t e _ e i n f u e g e n
    # Klasse       : Graph::GED::Graph
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Fgt eine Kante in den Graphen ein.
    # Parameter    : 1) Startecke
    #                2) Zielecke
    #                3) Optional Kantenform, sonst $self->kantenform()
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 08.02.2004 - CD - erstellt
    # 0.0.2 - 10.02.2004 - CD - neue Hilfsmethoden verwendet.
    #                           Umstellung auf die Kantenklasse.
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph::kante_einfuegen falscher Anzahl Parametern " . 
        "aufgerufen!"
        if scalar @_ < 2 or scalar @_ > 3;
    die "Graph::GED::Graph::kante_einfuegen aufgerufen, aber der Graph hat " .
        "keine Ecken!"
        unless $self->n() > 0;

    my $ea = shift;
    my $ee = shift;
    my $kf = shift || $self->kantenform();


    print "Graph::GED::Graph::kante_einfuegen($ea, $ee, $kf)\n" if $Debug;

    my $k = new Graph::GED::Kante;
    $k->a   ($ea);
    $k->e   ($ee);
    $k->form($kf);

    $self->fuege_kante_hinzu($k);

    $self->inc_m();

} # sub Graph::kante_einfuegen


sub kante_loeschen {
    #--------------------------------------------------------------------------
    # Methode      : k a n t e _ l o e s c h e n
    # Klasse       : Graph::GED::Graph
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Lscht die angegebene Kante aus dem Graphen.
    # Parameter    : Kante
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 08.02.2004 - CD - erstellt
    # 0.0.2 - 10.02.2004 - CD - neue Hilfsmethoden verwendet.
    #                           Umstellung auf die Kantenklasse.
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph::kante_loeschen mit falscher Anzahl Parametern " . 
        "aufgerufen!"
        if scalar @_ != 1;

    my $ki   = shift;

    print "Graph::GED::Graph::kante_loeschen($ki)\n" if $Debug;

    my $k = $self->entnehme_kante($ki);

    print "lsche Kante $ki ", $k->a(), " --- ", $k->e(), "\n"
          if $Debug>1;


    $self->dec_m();

} # sub Graph::kante_loeschen


sub clear {
    #--------------------------------------------------------------------------
    # Methode      : c l e a r
    # Klasse       : Graph::GED::Graph
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Lscht alle Ecken und Kanten.
    # Parameter    : keine
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 09.02.2003 - CD - Erstellt
    # 0.0.2 - 10.02.2004 - CD - neue Hilfsmethoden verwendet.
    #                           Umstellung auf die Kantenklasse.
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Graph::clear mit Parametern aufgerufen!"
        if scalar @_ > 0;

    print "Graph::GED::Graph::clear\n" if $Debug;

    #--------------------------------------------------------------------------
    # Ecken- und Kantenzahl auf Null setzen:
    #--------------------------------------------------------------------------
    $self->n(0);
    $self->m(0);

    #--------------------------------------------------------------------------
    # Kanten lschen:
    #--------------------------------------------------------------------------
    $self->loesche_kanten();

    #--------------------------------------------------------------------------
    # Basisfunktionalitt aufrufen:
    #--------------------------------------------------------------------------
    $self->SUPER::clear();
} # sub Graph::GED::Graph::clear


return 1;

__END__

=head1 NAME

Graph::GED::Graph.pm - a module for graphs

=head1 METHODS

=head2 class methods

=over

=item C<new>

    Creates an object of this class.

=back

=head2 object methods

=over

=item C<clone>

    Creates a cloned object of this graph.

=item C<n>

    Getter / Setter for the number of vertices

=item C<m>

    Getter / Setter for the number of edges

=item C<kantenform>

    Getter / Setter for the actual edge form

=item C<_kanten>

    Getter / Setter for the set of edges

=item C<inc_n>

    Increments n (the number of vertices).

=item C<dec_n>

    Decrements n (the number of vertices).

=item C<inc_m>

    Increments n (the number of edges).

=item C<dec_m>

    Decrements n (the number of edges).

=item C<setze_kanten>

    sets the set of edges

=item C<loesche_kanten>

    delets all edges

=item C<kante>

    Getter / Setter for one edge object.
    
=item C<entnehme_kante>

    Splices one edge out of the edge set and returns it.

=item C<fuege_kante_hinzu>

    Adds one edge to the edge set.

=item C<ecke_einfuegen>

    Adds one vertex to the graph.

=item C<ecke_loeschen>

    Deletes one vertex from the graph.

=item C<ecke_mergen>

    Merges on vertex away,

=item C<kante_existiert>

    Checks if the given edge is existing.

=item C<kante_einfuegen>

    Adds one edge to the graph.

=item C<kante_loeschen>

    Deletes one edge from the graph.

=item C<clear>

    Deletes all edges and vertices.

=back

=head1 COPYRIGHT

Copyright 2004-2005 Christian Duehl. All rights reserved.

This library is free software. You can redistribute it and/or
modify it under the same terms as perl itself.

=cut

